
from PyQt5 import QtWidgets, QtGui, QtCore
from libnam import control
from libabr import res, app as apps
import importlib

class Run (QtWidgets.QMainWindow):
    ictitle = None
    lbltitle = None
    model = control.read_record("model","/etc/gui")
    model = model.split("x")
    mw = int(model[0]) # width of model
    mh = int(model[1]) # height of model

    def setWindowTitle (self,title):
        self.lbltitle.clear()
        self.lbltitle.setText(title)

    def windowTitle (self):
        return self.lbltitle.text()

    def setWindowIcon (self,icon):
        self.icon = QtGui.QIcon(icon)
        self.ictitle.setPixmap(self.icon.pixmap(30))
        self.ictitle.setFixedSize(30, 30)

    def resize (self,w,h):
        self.w = w
        self.h = h
        self.setGeometry(int(self.mw / 2) - int(self.w / 2), int(self.mh / 2) - int(self.h / 2), self.w,self.h)  # Center of screen at all

    def __init__(self,mainApp,Desktop,ID):
        super(Run, self).__init__()
        self.mainApp = mainApp
        self.ID = ID

        ## Start id process ##
        apps.start (self.ID)

        ## Resize ##
        self.resize(1000,600)

        ## Get all databases ##
        self.default_icon = res.get(control.read_record("app.icon.default-icon", "/etc/gui"))
        self.bgcolor = control.read_record("app.title.bgcolor", "/etc/gui")
        self.fgcolor = control.read_record("app.title.fgcolor", "/etc/gui")
        self.fontsize = control.read_record("fontsize", "/etc/gui")

        self.setStyleSheet('background-color: white;border: none;')

        titlebar = QtWidgets.QMenuBar()
        titlebar.setMinimumSize(40,40)
        titlebar.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.setMenuBar(titlebar)

        ## Set Colors for TitleBar ##
        if not self.bgcolor == None and not self.fgcolor == None:
            titlebar.setStyleSheet("background-color: " + self.bgcolor + ";color: " + self.fgcolor + ";")
        elif not self.bgcolor == None:
            if self.bgcolor == "#FFFFFF" or self.bgcolor == "white":
                titlebar.setStyleSheet("background-color: " + self.bgcolor + ";color: black;")
            else:
                self.title.setStyleSheet("background-color: " + self.bgcolor + ";color: white;")
        elif not self.fgcolor == None:
            titlebar.setStyleSheet("background-color: #00BFFF;color: " + self.fgcolor)
        else:
            titlebar.setStyleSheet("background-color: #00BFFF;color: white;")

        ## Get icons ##
        if res.get("@string/direction") == "ltr":
            self.escapei = res.get(control.read_record("app.icon.escape-left", "/etc/gui"))
        elif res.get("@string/direction") == "rtl":
            self.escapei = res.get(control.read_record("app.icon.escape-right", "/etc/gui"))
        else:
            self.escapei = res.get(control.read_record("app.icon.escape-left", "/etc/gui"))

        ## 3 actions ##
        self.actionClose = QtWidgets.QAction()
        titlebar.addAction(self.actionClose)
        self.actionClose.setIcon(QtGui.QIcon(self.escapei))
        self.actionClose.triggered.connect(self.actClose)

        self.actionMax = QtWidgets.QAction()
        titlebar.addAction(self.actionMax)
        self.actionMax.setIcon(QtGui.QIcon(res.get(control.read_record("app.icon.max", "/etc/gui"))))
        self.actionMax.triggered.connect(self.actMax)

        self.actionMin = QtWidgets.QAction()
        titlebar.addAction(self.actionMin)
        self.actionMin.setIcon(QtGui.QIcon(res.get(control.read_record("app.icon.min", "/etc/gui"))))
        self.actionMin.triggered.connect(self.actMin)

        ## Corner widget ##
        self.title = QtWidgets.QWidget()
        self.titlelayout = QtWidgets.QHBoxLayout()
        self.ictitle = QtWidgets.QLabel()

        self.icon = QtGui.QIcon(QtGui.QIcon(res.get('@logo/app')))
        self.ictitle.setPixmap(self.icon.pixmap(30))
        self.ictitle.setFixedSize(30, 30)

        self.lbltitle = QtWidgets.QLabel('Widget')
        self.lbltitle.setStyleSheet('color: '+self.fgcolor+';')
        self.titlelayout.addWidget(self.lbltitle)
        self.titlelayout.addWidget(self.ictitle)
        self.title.setLayout(self.titlelayout)

        titlebar.setCornerWidget(self.title)
        mainApp = mainApp.MainApp(Desktop, self)
        self.setCentralWidget(mainApp)  # Run widget

    def actClose(self):
        apps.end(self.ID)
        self.close()

    def actMin(self):
        self.hide()

    maxf = False
    h = 0
    w = 0
    def actMax (self):
        if self.maxf==False:
            self.w = self.width()
            self.h = self.height()

            # Get taskbar location #

            self.setGeometry(0,0,self.mw,self.mh)
            self.maxf = True
            self.actionMax.setIcon(QtGui.QIcon(res.get(control.read_record("app.icon.max-min", "/etc/gui"))))
        else:
            self.maxf = False
            self.setGeometry(int(self.mw/2)-int(self.w/2), int(self.mh/2)-int(self.h/2),self.w,self.h) # Center of screen at all
            self.actionMax.setIcon(QtGui.QIcon(res.get(control.read_record("app.icon.max", "/etc/gui"))))

    ## Customize the window  (stackoverflow.com) ##
    def mousePressEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.__press_pos = event.pos()


    def mouseReleaseEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.__press_pos = None

    def mouseMoveEvent(self, event):
        if self.__press_pos:
            self.move(self.pos() + (event.pos() - self.__press_pos))