
from PyQt5 import QtWidgets, QtGui, QtCore, uic
from libnam import control, files, permissions, modules
from libabr import res, app as apps
import hashlib, app, sys,importlib

class Desktop (QtWidgets.QMainWindow):
    model = control.read_record("model", "/etc/gui")
    model = model.split("x")
    mw = int(model[0])  # width of model
    mh = int(model[1])  # height of model

    def __init__(self):
        super(Desktop, self).__init__()
        self.resize(self.mw,self.mh)

        ## Get distro database
        self.cs = files.readall("/proc/info/cs")
        self.cd = files.readall("/proc/info/cd")
        self.ver = files.readall("/proc/info/ver")
        self.setWindowTitle(self.cs + " " + self.ver + " (" + self.cd + ")")

################ App #############################
class MainApp (QtWidgets.QMainWindow):
    fontsize = control.read_record("fontsize", "/etc/gui")
    username = ""
    locale = control.read_record("locale","/etc/gui")
    def __init__(self):
        super(MainApp, self).__init__()

        ## Get username database
        username = control.read_record("username", "/tmp/su.tmp")
        code = control.read_record("code", "/tmp/su.tmp")

        self.username = username
        self.code = code

        self.w = Desktop()
        self.layout().addWidget(self.w)
        uic.loadUi(res.get('@widget/desktop'), self.w)

        ## Default bgcolor check ##
        bgcolor = None
        if not username == "guest":
            bgcolor = control.read_record("desktop.bgcolor", "/etc/users/" + username)
        if bgcolor == None:
            bgcolor = control.read_record("desktop.bgcolor", "/etc/gui")
        background = None
        if not username == "guest":
            background = res.get(control.read_record("desktop.background", "/etc/users/" + username))
        if background == None:
            background = res.get(control.read_record("desktop.background", "/etc/gui"))

        if not bgcolor == None:
            self.w.setStyleSheet("background-color: " + bgcolor + ";")
        elif not background == None:
            self.w.setStyleSheet("background-image: url(" + background + ");")
        else:
            self.w.setStyleSheet("background-color: blue;")

        ## Remove temporary files
        if files.isfile("/tmp/username.tmp"): files.remove("/tmp/username.tmp")
        if files.isfile("/tmp/su.tmp"): files.remove("/tmp/su.tmp")

        ## Switch this selected user
        files.write("/proc/info/su", username)
        permissions.user = username

        ## Check user ##
        if not username == "guest":
            if not files.isfile("/etc/users/" + username):
                exit(0)

            user = control.read_record("username", "/etc/users/" + username)
            hashname = hashlib.sha3_256(
                str(username).encode()).hexdigest()

            if not user == hashname:
                exit(0)

            password = control.read_record("code", "/etc/users/" + username)
            hashcode = hashlib.sha3_512(
                str(code).encode()).hexdigest()

            if not password == hashcode:
                exit(0)
        else:
            enable_gui = control.read_record("enable_gui", "/etc/guest")
            if enable_gui == "No": exit(0)

        ## Desktop /desk files extract ##

        deskdirs = control.read_list("/etc/deskdirs")

        if username == "root":
            for i in deskdirs:
                if not files.isdir("/root/" + i): files.mkdir("/root/" + i)
        else:
            if not files.isdir("/desk/" + username): files.mkdir("/desk/" + username)
            for i in deskdirs:
                if not files.isdir("/desk/" + username + "/" + i): files.mkdir("/desk/" + username + "/" + i)

        ## Create pwd for this user
        if username == "root":
            files.write("/proc/info/pwd", "/root")
        else:
            files.write("/proc/info/pwd", "/desk/" + username)

        ## Toolbar ##
        self.taskbar = QtWidgets.QToolBar()
        self.taskbar.setMinimumHeight(50)
        self.taskbar.setMinimumWidth(50)
        self.addToolBar(self.taskbar)

        '''## App button ##
        self.btnApp = QtWidgets.QToolButton()
        self.btnApp.setIcon(QtGui.QIcon(res.get('@logo/app')))
        self.btnApp.setMinimumSize(50, 50)
        self.taskbar.addWidget(self.btnApp)'''

        #self.btnApp.clicked.connect(self.clickApp)

        ## Set btnApp size
        if not self.username == "guest":
            btnApp_size = control.read_record("desktop.taskbar.size", "/etc/users/" + self.username)
        else:
            btnApp_size = None

        if btnApp_size == None:
            btnApp_size = control.read_record("desktop.taskbar.size", "/etc/gui")
            if btnApp_size == None:
                btnApp_size = 50
            else:
                btnApp_size = int(btnApp_size)

        ## Applications ##
        applist = files.list("/usr/share/applications")
        for i in applist:
            self.btnApp = QtWidgets.QToolButton()
            self.btnApp.setObjectName(i)
            self.btnApp.setMinimumSize(btnApp_size,btnApp_size)
            self.btnApp.setIconSize(QtCore.QSize(btnApp_size,btnApp_size))
            self.taskbar.addWidget(self.btnApp)

            self.btnApp.clicked.connect(self.AppRunner)
            self.shortcut = control.read_record("shortcut", "/usr/share/applications/" + i)
            self.comment = control.read_record("comment[" + self.locale + "]", "/usr/share/applications/" + i)
            self.name_ = control.read_record("name[" + self.locale + "]", "/usr/share/applications/" + i)
            self.icon = res.get(control.read_record("icon", "/usr/share/applications/" + i))

            ## Set Icon ##
            title_icon = control.read_record("app.icon", "/usr/share/applications/" + i)
            if title_icon == None:
                title_icon = control.read_record("app.icon", "/etc/gui")
                if not title_icon == None:
                    self.btnApp.setIcon(title_icon)
            else:
                self.btnApp.setIcon(title_icon)

            if not self.shortcut == None:
                self.btnApp.setShortcut(self.shortcut)
            if not self.comment == None:
                self.btnApp.setStatusTip(self.comment)
            if not self.name_ == None:
                self.btnApp.setText(self.name_)
            if not self.icon == None:
                self.btnApp.setIcon(QtGui.QIcon(self.icon))

        ## Set taskbar location ##
        if not self.username == "guest":
            taskbar_location = control.read_record("desktop.taskbar.location", "/etc/users/" + self.username)
        else:
            taskbar_location = None
        if taskbar_location == None:
            taskbar_location = control.read_record("desktop.taskbar.location", "/etc/gui")
            if taskbar_location == None:
                taskbar_location = "left"

        if taskbar_location == "bottom":
            self.addToolBar(QtCore.Qt.BottomToolBarArea, self.taskbar)
        elif taskbar_location == "left":
            self.addToolBar(QtCore.Qt.LeftToolBarArea, self.taskbar)
        elif taskbar_location == "right":
            self.addToolBar(QtCore.Qt.RightToolBarArea, self.taskbar)
        elif taskbar_location == "top":
            self.addToolBar(QtCore.Qt.TopToolBarArea, self.taskbar)
        else:
            self.addToolBar(QtCore.Qt.LeftToolBarArea, self.taskbar)

        ## Set taskbar size
        if not self.username == "guest":
            taskbar_size = control.read_record("desktop.taskbar.size", "/etc/users/" + self.username)
        else:
            taskbar_size = None

        if taskbar_size == None:
            taskbar_size = control.read_record("desktop.taskbar.size", "/etc/gui")
            if taskbar_size == None:
                taskbar_size = 50
            else:
                taskbar_size = int(taskbar_size)

        self.taskbar.setMinimumWidth(taskbar_size)
        self.taskbar.setMinimumHeight(taskbar_size)

        ## Set Taskbar color ##
        if not self.username == "guest":
            taskbar_color = control.read_record("desktop.taskbar.color", "/etc/users/" + self.username)
        else:
            taskbar_color = None
        if taskbar_color == None:
            taskbar_color = control.read_record("desktop.taskbar.color", "/etc/gui")
            if taskbar_color == None:
                taskbar_color = "#123456"

        self.taskbar.setStyleSheet("background-color: "+taskbar_color+";")

        ## Set Taskbar lock ##
        if not self.username == "guest":
            taskbar_locked = control.read_record("desktop.taskbar.locked", "/etc/users/" + self.username)
        else:
            taskbar_locked = None
        if taskbar_locked == None:
            taskbar_locked = control.read_record("desktop.taskbar.locked", "/etc/gui")
            if taskbar_locked == None:
                taskbar_locked = "Yes"

        if taskbar_locked=="Yes":
            self.taskbar.setMovable(False)
        else:
            self.taskbar.setMovable(True)

    ## Run a widget ##
    app_name = None

    def Run (self,widget):
        self.w.layout().addWidget(app.Run(widget, self.w,self.app_name))

    def AppRunner(self):
        sender = self.sender().objectName()
        exec = control.read_record("app", "/usr/share/applications/" + sender)
        self.app_name = sender.replace('.desk','') # Send id
        x = modules.import_module (exec)
        self.Run(x) # Run the widget

w = MainApp()
w.showFullScreen()